@@ -3,15 +3,22 @@ Dataclasses encoding response data from Dune API.
 """
 from __future__ import annotations
 
+import logging.config
 from dataclasses import dataclass
 from datetime import datetime
 from enum import Enum
-from typing import Optional, Any, Union
+from typing import Optional, Any, Union, List, Dict
 
 from dateutil.parser import parse
 from dune_client.types import DuneRecord
 
 
+log = logging.getLogger(__name__)
+logging.basicConfig(
+    format="%(asctime)s %(levelname)s %(name)s %(message)s", level=logging.INFO
+)
+
+
 class DuneError(Exception):
     """Possibilities seen so far
     {'error': 'invalid API Key'}
@@ -20,8 +27,10 @@ class DuneError(Exception):
     {'error': 'The requested execution ID (ID: Wonky Job ID) is invalid.'}
     """
 
-    def __init__(self, data: dict[str, str], response_class: str):
-        super().__init__(f"Can't build {response_class} from {data}")
+    def __init__(self, data: dict[str, str], response_class: str, err: KeyError):
+        error_message = f"Can't build {response_class} from {data}"
+        log.error(f"{error_message} due to KeyError: {err}")
+        super().__init__(error_message)
 
 
 class ExecutionState(Enum):
@@ -57,21 +66,26 @@ class TimeData:
     """A collection of all timestamp related values contained within Dune Response"""
 
     submitted_at: datetime
-    # For some reason... this field isn't always returned.
-    expires_at: Optional[datetime]
-    execution_started_at: datetime
+    execution_started_at: Optional[datetime]
     execution_ended_at: Optional[datetime]
+    # Expires only exists when we have result data
+    expires_at: Optional[datetime]
+    # only exists for cancelled executions
+    cancelled_at: Optional[datetime]
 
     @classmethod
     def from_dict(cls, data: dict[str, Any]) -> TimeData:
         """Constructor from dictionary. See unit test for sample input."""
-        end = data.get("execution_ended_at", None)
-        expires = data.get("expires_at", None)
+        start = data.get("execution_started_at")
+        end = data.get("execution_ended_at")
+        expires = data.get("expires_at")
+        cancelled = data.get("cancelled_at")
         return cls(
             submitted_at=parse(data["submitted_at"]),
             expires_at=None if expires is None else parse(expires),
-            execution_started_at=parse(data["execution_started_at"]),
+            execution_started_at=None if start is None else parse(start),
             execution_ended_at=None if end is None else parse(end),
+            cancelled_at=None if cancelled is None else parse(cancelled),
         )
 
 
@@ -85,14 +99,20 @@ class ExecutionStatusResponse:
     query_id: int
     state: ExecutionState
     times: TimeData
+    queue_position: Optional[int]
+    # this will be present when the query execution completes
+    result_metadata: Optional[ResultMetadata]
 
     @classmethod
-    def from_dict(cls, data: dict[str, str]) -> ExecutionStatusResponse:
+    def from_dict(cls, data: dict[str, Any]) -> ExecutionStatusResponse:
         """Constructor from dictionary. See unit test for sample input."""
+        dct: Optional[MetaData] = data.get("result_metadata")
         return cls(
             execution_id=data["execution_id"],
             query_id=int(data["query_id"]),
+            queue_position=data.get("queue_position"),
             state=ExecutionState(data["state"]),
+            result_metadata=ResultMetadata.from_dict(dct) if dct else None,
             times=TimeData.from_dict(data),  # Sending the entire data dict
         )
 
@@ -106,6 +126,9 @@ class ResultMetadata:
     column_names: list[str]
     result_set_bytes: int
     total_row_count: int
+    datapoint_count: int
+    pending_time_millis: int
+    execution_time_millis: int
 
     @classmethod
     def from_dict(cls, data: dict[str, int | list[str]]) -> ResultMetadata:
@@ -113,15 +136,21 @@ class ResultMetadata:
         assert isinstance(data["column_names"], list)
         assert isinstance(data["result_set_bytes"], int)
         assert isinstance(data["total_row_count"], int)
+        assert isinstance(data["datapoint_count"], int)
+        assert isinstance(data["pending_time_millis"], int)
+        assert isinstance(data["execution_time_millis"], int)
         return cls(
             column_names=data["column_names"],
-            result_set_bytes=int(data["result_set_bytes"]),
-            total_row_count=int(data["total_row_count"]),
+            result_set_bytes=data["result_set_bytes"],
+            total_row_count=data["total_row_count"],
+            datapoint_count=data["datapoint_count"],
+            pending_time_millis=data["pending_time_millis"],
+            execution_time_millis=data["execution_time_millis"],
         )
 
 
-RowData = list[dict[str, str]]
-MetaData = dict[str, Union[int, list[str]]]
+RowData = List[Dict[str, str]]
+MetaData = Dict[str, Union[int, List[str]]]
 
 
 @dataclass
@@ -142,7 +171,7 @@ class ExecutionResult:
         )
 
 
-ResultData = dict[str, Union[RowData, MetaData]]
+ResultData = Dict[str, Union[RowData, MetaData]]
 
 
 @dataclass
@@ -155,7 +184,8 @@ class ResultsResponse:
     query_id: int
     state: ExecutionState
     times: TimeData
-    result: ExecutionResult
+    # optional because it will only be present when the query execution completes
+    result: Optional[ExecutionResult]
 
     @classmethod
     def from_dict(cls, data: dict[str, str | int | ResultData]) -> ResultsResponse:
